﻿Imports System.ComponentModel
Imports System.Windows.Forms.Design
Imports System.Drawing.Drawing2D


Public Class NABiButton
    Inherits Infragistics.Win.Misc.UltraButton
    Implements iNABiControlProperty

#Region " Private Fields "

    Private meUseFontStyle As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE

    Private meButtonAutoSize As ENUM_TRUEFALSE = ENUM_TRUEFALSE.TRUE
    Private meButtonType As ENUM_BUTTONTYPE = ENUM_BUTTONTYPE.DEFALUT
    Private Const BTN_HEIGHT_SIZE As Integer = 28

    Private moUIControlsCommonCLS As NABi.UI.Common.UIControlCommonCLS

#End Region

#Region " Constructors "

    ''' <summary>
    ''' 생성자
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()

        Dim bDesignMode As Boolean = IIf(LicenseManager.UsageMode = LicenseUsageMode.Designtime, True, False)

        'designmode 가 아닌 경우에만 UIControlsCommon 의 WCF DB 연결을 수행
        moUIControlsCommonCLS = NABi.UI.Common.UIControlCommonCLS.NABiGetInstance(bDesignMode)

    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' button 종류 설정
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Button 종류 설정")> _
    Public Property NABiButtonType() As ENUM_BUTTONTYPE
        Get
            Return meButtonType
        End Get
        Set(ByVal Value As ENUM_BUTTONTYPE)
            meButtonType = Value
            NABiSetButtonType()
            Me.Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' button size 자동 조절
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Text 길이에 따라 button size 자동조절")> _
    Public Property NABiButtonAutoSize() As ENUM_TRUEFALSE
        Get
            Return meButtonAutoSize
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meButtonAutoSize = Value
            If meButtonAutoSize = ENUM_TRUEFALSE.TRUE Then
                NABiSetButtonSize()
            End If
            Me.Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' button Text Overrides.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    Public Overrides Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
            If meButtonAutoSize = ENUM_TRUEFALSE.TRUE Then
                NABiSetButtonSize()
            End If
        End Set
    End Property

    ''' <summary>
    ''' button Font style 자동 조절
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Font style 설정 (맑은고딕/9pt)")> _
    Public Property NABiUseFontStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseFontStyle
        Get
            Return meUseFontStyle
        End Get
        Set(value As ENUM_TRUEFALSE)
            meUseFontStyle = value
            NABiSetFontStyle(meUseFontStyle)
            Me.Invalidate()
        End Set
    End Property

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseNABiStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseNABiStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataProtect As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiDataProtect

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataType As ENUM_PROTECTDATA Implements iNABiControlProperty.NABiDataType

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindDTL As ENUM_BINDDETAIL Implements iNABiControlProperty.NABiBindDTL

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindMaster As ENUM_BINDMASTER Implements iNABiControlProperty.NABiBindMaster

#End Region

#Region " Private Methods "

    ''' <summary>
    ''' 버튼 사이즈 자동 조절
    ''' </summary>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    Private Sub NABiSetButtonSize()

        Dim oButtonSize As Size = Me.Size
        Dim oSize As SizeF
        Dim oGraphics As Graphics

        If meButtonAutoSize = ENUM_TRUEFALSE.TRUE Then

            oGraphics = Me.CreateGraphics

            Using oGraphics

                oSize = oGraphics.MeasureString(Me.Text, Me.Font)

                oButtonSize.Width = oSize.Width + 11
                oButtonSize.Height = BTN_HEIGHT_SIZE


            End Using

        End If

        NABiSetButtonType()

        Me.Size = oButtonSize

    End Sub

    ''' <summary>
    ''' 버튼 Type 설정
    ''' </summary>
    ''' <remarks>
    ''' 2015-07-07 민유진 Created
    ''' </remarks>
    Private Sub NABiSetButtonType()

        Me.AutoSize = False

        Select Case meButtonType
            Case ENUM_BUTTONTYPE.DEFALUT
                Me.UseOsThemes = Infragistics.Win.DefaultableBoolean.False
                Me.Appearance.BorderColor = moUIControlsCommonCLS.NABiRemoveButtonBorderColor
                Me.ButtonStyle = Infragistics.Win.UIElementButtonStyle.Flat
                Me.StyleLibraryName = Nothing

            Case ENUM_BUTTONTYPE.NONE
                Me.StyleLibraryName = "None"

            Case ENUM_BUTTONTYPE.SAVE, ENUM_BUTTONTYPE.CONFIRM
                Me.StyleLibraryName = "None"

                'Normal
                Me.Appearance.BackColor = moUIControlsCommonCLS.NABiSaveButtonBackColor
                Me.Appearance.BackColor2 = moUIControlsCommonCLS.NABiSaveButtonBackColor2
                Me.Appearance.BorderColor = moUIControlsCommonCLS.NABiSaveButtonBorderColor
                Me.Appearance.ForeColor = moUIControlsCommonCLS.NABiSaveButtonForeColor

                'Press
                Me.PressedAppearance.BackColor = moUIControlsCommonCLS.NABiSaveButtonPressedBackColor
                Me.PressedAppearance.BackColor2 = moUIControlsCommonCLS.NABiSaveButtonPressedBackColor2

                'Over
                Me.UseHotTracking = Infragistics.Win.DefaultableBoolean.True
                Me.HotTrackAppearance.BackColor = moUIControlsCommonCLS.NABiSaveButtonHotTrackBackColor
                Me.HotTrackAppearance.BackColor2 = moUIControlsCommonCLS.NABiSaveButtonHotTrackBackColor2

                Me.ButtonStyle = Infragistics.Win.UIElementButtonStyle.Flat
                Me.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True

            Case ENUM_BUTTONTYPE.REMOVE
                Me.StyleLibraryName = "None"

                'Normal
                Me.Appearance.BackColor = moUIControlsCommonCLS.NABiRemoveButtonBackColor
                Me.Appearance.BackColor2 = moUIControlsCommonCLS.NABiRemoveButtonBackColor2
                Me.Appearance.BorderColor = moUIControlsCommonCLS.NABiRemoveButtonBorderColor
                Me.Appearance.ForeColor = moUIControlsCommonCLS.NABiRemoveButtonForeColor

                'Press
                Me.PressedAppearance.BackColor = moUIControlsCommonCLS.NABiRemoveButtonPressedBackColor
                Me.PressedAppearance.BackColor2 = moUIControlsCommonCLS.NABiRemoveButtonPressedBackColor2

                'Over
                Me.UseHotTracking = Infragistics.Win.DefaultableBoolean.True
                Me.HotTrackAppearance.BackColor = moUIControlsCommonCLS.NABiRemoveButtonHotTrackBackColor
                Me.HotTrackAppearance.BackColor2 = moUIControlsCommonCLS.NABiRemoveButtonHotTrackBackColor2

                Me.ButtonStyle = Infragistics.Win.UIElementButtonStyle.Flat
                Me.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True

        End Select

        Select Case meButtonType
            Case ENUM_BUTTONTYPE.SAVE, ENUM_BUTTONTYPE.REMOVE, ENUM_BUTTONTYPE.CONFIRM
                If Not DesignMode Then
                    Me.Region = RoundControlCLS.RoundedRegion(Me.ClientRectangle, 1, RoundControlCLS.Corner.None)
                End If

                Me.UseOsThemes = Infragistics.Win.DefaultableBoolean.False

                Me.Appearance.BackGradientStyle = Infragistics.Win.GradientStyle.Vertical
                Me.PressedAppearance.BackGradientStyle = Infragistics.Win.GradientStyle.Vertical
                Me.HotTrackAppearance.BackGradientStyle = Infragistics.Win.GradientStyle.Vertical

                NABiSetFontStyle(meUseFontStyle)

                moUIControlsCommonCLS.NABiSetButtonColor()
        End Select


    End Sub

    ''' <summary>
    ''' Font Style 자동 설정
    ''' </summary>
    ''' <param name="peFontStyle"></param>
    ''' <remarks></remarks>
    Public Sub NABiSetFontStyle(peFontStyle As ENUM_TRUEFALSE) Implements iNABiControlProperty.NABiSetFontStyle
        If peFontStyle = ENUM_TRUEFALSE.TRUE Then
            Me.Appearance.FontData.Name = NABI_FONTNAME
            Me.Appearance.FontData.SizeInPoints = NABI_FONTSIZE
        ElseIf peFontStyle = ENUM_TRUEFALSE.FALSE Then
            Me.Appearance.StyleLibraryName = Nothing
            Me.Appearance.FontData.Name = Nothing
            Me.Appearance.FontData.SizeInPoints = Nothing
        End If
    End Sub

#End Region

End Class
