﻿Imports System.ComponentModel

Public Class NABiDateTimeEditor
    Inherits Infragistics.Win.UltraWinEditors.UltraDateTimeEditor
    Implements iNABiControlProperty

#Region " Private Fields "

    Private meUseFontStyle As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE
    Private meDateTimeType As ENUM_DATETYPE = ENUM_DATETYPE.NONE
    Private meUseNullType As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE

    ''NABi font
    'Private Const NABI_FONTSIZE As Single = 9
    'Private Const NABI_FONTNAME As String = "맑은 고딕"

#End Region

#Region " Properties "

    ''' <summary>
    ''' ComboEditor Font style 자동 조절
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Font style 설정 (맑은고딕/9pt)")> _
    Public Property NABiUseFontStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseFontStyle
        Get
            Return meUseFontStyle
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meUseFontStyle = Value
            NABiSetFontStyle(meUseFontStyle)
            Me.Invalidate()
        End Set
    End Property

    <CategoryAttribute("NABi Design"), Browsable(True), Description("DateTime Format 설정")> _
    Public Property NABiDateFormatType() As ENUM_DATETYPE
        Get
            Return meDateTimeType
        End Get
        Set(ByVal Value As ENUM_DATETYPE)
            meDateTimeType = Value
            If meDateTimeType <> ENUM_DATETYPE.NONE Then
                NABiSetDateType()
            End If
            Me.Invalidate()
        End Set
    End Property

    <CategoryAttribute("NABi Design"), Browsable(True), Description("DateTime 현재 시간 설정")> _
    Public Property NABiUseNullType() As ENUM_TRUEFALSE
        Get
            Return meUseNullType
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meUseNullType = Value
            NABiSetNullType(meUseNullType)
            Me.Invalidate()
        End Set
    End Property


    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseNABiStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseNABiStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataProtect As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiDataProtect

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataType As ENUM_PROTECTDATA Implements iNABiControlProperty.NABiDataType

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindDTL As ENUM_BINDDETAIL Implements iNABiControlProperty.NABiBindDTL

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindMaster As ENUM_BINDMASTER Implements iNABiControlProperty.NABiBindMaster

#End Region

#Region " Private Methods "

    ''' <summary>
    ''' NABi Font style 설정
    ''' </summary>
    ''' <param name="peFontStyle"></param>
    ''' <remarks></remarks>
    Public Sub NABiSetFontStyle(peFontStyle As ENUM_TRUEFALSE) Implements iNABiControlProperty.NABiSetFontStyle
        If peFontStyle = ENUM_TRUEFALSE.TRUE Then
            Me.Appearance.FontData.Name = NABI_FONTNAME
            Me.Appearance.FontData.SizeInPoints = NABI_FONTSIZE
        ElseIf peFontStyle = ENUM_TRUEFALSE.FALSE Then

        End If
    End Sub

    ''' <summary>
    ''' DateTime Type에 따른 MaskInput 설정
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub NABiSetDateType()

        If DesignMode Then

            Dim sMaskInput As String = Nothing

            Dim iMonthSize As Integer = 80
            Dim iDateSize As Integer = 100
            Dim iDateTimeSize As Integer = 135
            Dim iDateTimeSec As Integer = 150
            Dim iDateTimeWeek As Integer = 170

            Select Case meDateTimeType
                Case ENUM_DATETYPE.MONTH
                    sMaskInput = "{LOC}yyyy/mm"
                    Me.FormatString = Nothing
                    Me.Size = New Size(iMonthSize, Me.Size.Height)

                Case ENUM_DATETYPE.DATE
                    sMaskInput = "{LOC}yyyy/mm/dd"
                    Me.FormatString = Nothing
                    Me.Size = New Size(iDateSize, Me.Size.Height)

                Case ENUM_DATETYPE.DATETIME
                    sMaskInput = "{LOC}yyyy/mm/dd hh:mm"
                    Me.FormatString = Nothing
                    Me.Size = New Size(iDateTimeSize, Me.Size.Height)

                Case ENUM_DATETYPE.DATETIMESEC
                    sMaskInput = "{LOC}yyyy/mm/dd hh:mm:ss"
                    Me.FormatString = Nothing
                    Me.Size = New Size(iDateTimeSec, Me.Size.Height)

                Case ENUM_DATETYPE.DATEWEEKDAY
                    sMaskInput = "yyyy년mm월dd일"
                    Me.FormatString = "yyyy년 MM월 dd일 dddd"
                    Me.Size = New Size(iDateTimeWeek, Me.Size.Height)

            End Select

            Me.MaskInput = sMaskInput
            NABiSetNullType(meUseNullType)

        End If

    End Sub

    ''' <summary>
    ''' Value값에 Null 값 허용시 Value 값 및 색상 설정
    ''' </summary>
    ''' <param name="peUseNullType"></param>
    ''' <remarks></remarks>
    Private Sub NABiSetNullType(ByVal peUseNullType As ENUM_TRUEFALSE)

        If DesignMode Then

            Dim sNullText As String = ""

            If peUseNullType = ENUM_TRUEFALSE.TRUE Then
                'Null 허용 및 사용
                Me.Nullable = True
                Me.Appearance.ForeColor = Color.Silver

                Select Case meDateTimeType
                    Case ENUM_DATETYPE.MONTH
                        sNullText = "____-__"

                    Case ENUM_DATETYPE.DATE
                        sNullText = "____-__-__"

                    Case ENUM_DATETYPE.DATETIME
                        sNullText = "____-__-__ __:__"

                    Case ENUM_DATETYPE.DATETIMESEC
                        sNullText = "____-__-__ __:__:__"

                End Select

                Me.NullText = sNullText
                Me.Value = Nothing

            ElseIf peUseNullType = ENUM_TRUEFALSE.FALSE Then
                'Null허용 안함
                Me.Nullable = False
                Me.Appearance.ForeColor = Color.Black

            End If

        End If

    End Sub

#End Region

#Region " Events "

    ''' <summary>
    ''' meUseNullType가 True일때 ForeColor 설정 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub NABiDateTimeEditor_ValueChanged(sender As Object, e As EventArgs) Handles Me.ValueChanged
        If meUseNullType = ENUM_TRUEFALSE.TRUE Then
            If Me.Value = Nothing Then
                Me.Appearance.ForeColor = Color.Silver
            Else
                Me.Appearance.ForeColor = Color.Black
            End If
        Else
            Me.Appearance.ForeColor = Color.Black
        End If
    End Sub

#End Region

End Class
