﻿Imports System.ComponentModel
Imports System.Text
Imports Infragistics.Win
Imports Infragistics.Win.Misc

Imports NABi.UI.Common.UIControlCommonCLS

Public Class NABiLabel
    Inherits Infragistics.Win.Misc.UltraLabel
    Implements iNABiControlProperty


#Region " Private Fields "

    '라벨 font style 사용여부
    Private meUseFontStyle As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE

    Private meLabelStyle As ENUM_LABELSTYLE = ENUM_LABELSTYLE.LABEL '기본라벨 및 움직이는 라벨

    Private moMoveLabelDrawFilter As NABiLabelDrawFilter  '움직이는 라벨 객체

    Private meLabelType As ENUM_LABELTYPE = ENUM_LABELTYPE.NONE   '라벨형식 및 정렬

    ''라벨 Data값에 따른 보안설정
    Private meDataType As ENUM_PROTECTDATA = ENUM_PROTECTDATA.NONE
    Private meDataProtect As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE


    'Label 크기에 따라 자동으로 Font size 변경 여부 혹은 
    Private meAutoFontSize As ENUM_LABELAUTOSIZE = ENUM_LABELAUTOSIZE.NONE

    ''NABi font
    'Private Const NABI_FONTSIZE As Single = 9
    'Private Const NABI_FONTNAME As String = "맑은 고딕"

    Private meLabelHelpColor As ENUM_COMMONCOLOR = ENUM_COMMONCOLOR.NONE

    Private moUIControlsCommon As NABi.UI.Common.UIControlCommonCLS

#End Region

#Region " Properties "

    <CategoryAttribute("NABi Design"), Browsable(True), Description("Common Help Color 설정")> _
    Public Property NABiHelpColor As ENUM_COMMONCOLOR
        Get
            Return meLabelHelpColor
        End Get
        Set(value As ENUM_COMMONCOLOR)
            meLabelHelpColor = value
            NABiSetHelpColor(meLabelHelpColor)
            Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' Label Font size 자동 조절
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-23 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Label크기에 따른 자동 font size 설정")> _
    Public Property NABiUseAutoSize() As ENUM_LABELAUTOSIZE
        Get
            Return meAutoFontSize
        End Get
        Set(ByVal Value As ENUM_LABELAUTOSIZE)
            meAutoFontSize = Value
            If meAutoFontSize = ENUM_LABELAUTOSIZE.FONTAUTOSIZE Then
                'NABiUseFontStyle은 false로 변경해준다.
                NABiUseFontStyle = ENUM_TRUEFALSE.FALSE
                Me.AutoSize = False
            ElseIf meAutoFontSize = ENUM_LABELAUTOSIZE.AUTOSIZE Then
                Me.AutoSize = True
            End If
            Me.Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' Label Type 및 정렬 선택
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Label Type 및 정렬 선택")> _
    Public Property NABiLabelType() As ENUM_LABELTYPE
        Get
            Return meLabelType
        End Get
        Set(ByVal Value As ENUM_LABELTYPE)
            meLabelType = Value
            NABiSetLabelType(meLabelType)
            Me.Invalidate()
        End Set
    End Property

    ''' <summary>
    ''' Label Style 선택
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' 2015-06-18 민유진 Created
    ''' </remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("기본Label or 움직이는 Label 선택")> _
    Public Property NABiLabelStyle() As ENUM_LABELSTYLE
        Get
            Return meLabelStyle
        End Get
        Set(ByVal Value As ENUM_LABELSTYLE)
            meLabelStyle = Value
            NabiSetLabelStyle()
            Me.Invalidate()
        End Set
    End Property


    <CategoryAttribute("NABi Design"), Browsable(True), Description("Font style 설정 (맑은고딕/9pt)")>
    Public Property NABiUseFontStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseFontStyle
        Get
            Return meUseFontStyle
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meUseFontStyle = Value
            NABiSetFontStyle(meUseFontStyle)
            Me.Invalidate()
        End Set
    End Property


    <CategoryAttribute("NABi Design"), Browsable(True), Description("Label 보안 설정 데이터")>
    Public Property NABiDataType As ENUM_PROTECTDATA Implements iNABiControlProperty.NABiDataType
        Get
            Return meDataType
        End Get
        Set(ByVal Value As ENUM_PROTECTDATA)
            meDataType = Value
            Me.Invalidate()
        End Set
    End Property


    <CategoryAttribute("NABi Design"), Browsable(True), Description("Label 보안 설정 여부")>
    Public Property NABiDataProtect As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiDataProtect
        Get
            Return meDataProtect
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meDataProtect = Value
            Me.Invalidate()
        End Set
    End Property

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseNABiStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseNABiStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindDTL As ENUM_BINDDETAIL Implements iNABiControlProperty.NABiBindDTL

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindMaster As ENUM_BINDMASTER Implements iNABiControlProperty.NABiBindMaster

#End Region

#Region " Constructors "

    ''' <summary>
    ''' 생성자
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()

        Dim bDesignMode As Boolean = IIf(LicenseManager.UsageMode = LicenseUsageMode.Designtime, True, False)

        '최한표20150702 추가 - designmode 가 아닌 경우에만 UIControlsCommon 의 WCF DB 연결을 수행
        moUIControlsCommon = NABi.UI.Common.UIControlCommonCLS.NABiGetInstance(bDesignMode)

    End Sub

#End Region

#Region " Private Methods "

    Private Property meUseAutoFontSizeChange As ENUM_TRUEFALSE

    Private Sub NabiSetLabelStyle()

        Select Case meLabelStyle
            Case ENUM_LABELSTYLE.MOVELABEL
                Me.HotTrackAppearance.Cursor = Cursors.Hand
                Me.HotTrackAppearance.ForeColor = Color.Blue
                Me.Size = New Size(Me.Width, 16)
                moMoveLabelDrawFilter = New NABiLabelDrawFilter(Me)
                Me.DrawFilter = moMoveLabelDrawFilter

            Case Else
                Me.HotTrackAppearance.Cursor = Cursors.Default
                Me.HotTrackAppearance.ForeColor = Nothing
                moMoveLabelDrawFilter = Nothing
                Me.DrawFilter = Nothing
                NABiSetLabelType(meLabelType)

        End Select

    End Sub

    ''' <summary>
    ''' Title성 Label과 Value성 Label 구분 및 정렬
    ''' </summary>
    ''' <remarks>
    ''' 2015-06-17 민유진 Created
    ''' </remarks>
    Private Sub NABiSetLabelType(ByVal peLabelType As ENUM_LABELTYPE)

        If peLabelType <> ENUM_LABELTYPE.NONE Then

            Me.Appearance.TextVAlign = Infragistics.Win.VAlign.Middle

            If meLabelStyle = ENUM_LABELSTYLE.LABEL Then

                Select Case meLabelType
                    Case ENUM_LABELTYPE.TITLE_LEFT, ENUM_LABELTYPE.VALUE_LEFT
                        Me.Appearance.TextHAlign = Infragistics.Win.HAlign.Left
                    Case ENUM_LABELTYPE.TITLE_RIGHT, ENUM_LABELTYPE.VALUE_RIGHT
                        Me.Appearance.TextHAlign = Infragistics.Win.HAlign.Right
                    Case ENUM_LABELTYPE.TITLE_MIDDLE, ENUM_LABELTYPE.VALUE_MIDDLE
                        Me.Appearance.TextHAlign = Infragistics.Win.HAlign.Center

                End Select

                Select Case meLabelType
                    'Label이 Title인 경우
                    Case ENUM_LABELTYPE.TITLE_LEFT, ENUM_LABELTYPE.TITLE_RIGHT, ENUM_LABELTYPE.TITLE_MIDDLE
                        Me.Appearance.ForeColor = Color.FromArgb(60, 60, 60)
                        Me.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True
                    Case Else
                        Me.Appearance.ForeColor = Color.Black
                        Me.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.False
                End Select

            End If

        End If

    End Sub

    ' ''' <summary>
    ' ''' NABi Fontsize 설정
    ' ''' </summary>
    ' ''' <remarks>
    ' ''' 2015-06-23 민유진 Created
    ' ''' </remarks>
    'Private Sub NABiSetLabelFontStyle(peUseLabelFontStyle As ENUM_TRUEFALSE)
    '    If peUseLabelFontStyle = ENUM_TRUEFALSE.TRUE Then
    '        Me.Appearance.FontData.Name = NABI_FONTNAME
    '        Me.Appearance.FontData.SizeInPoints = NABI_FONTSIZE
    '    ElseIf peUseLabelFontStyle = ENUM_TRUEFALSE.FALSE Then
    '        Me.Appearance.FontData.Name = Nothing
    '        Me.Appearance.FontData.SizeInPoints = Nothing
    '    End If
    'End Sub


    Public Sub NABiSetFontStyle(peFontStyle As ENUM_TRUEFALSE) Implements iNABiControlProperty.NABiSetFontStyle
        If peFontStyle = ENUM_TRUEFALSE.TRUE Then
            Me.Appearance.FontData.Name = NABI_FONTNAME
            Me.Appearance.FontData.SizeInPoints = NABI_FONTSIZE
        End If
    End Sub


    ''' <summary>
    ''' DesignMode인 경우 style 변경 /  Runtime 시 BackColor설정
    ''' </summary>
    ''' <param name="peLabelHelpColor"></param>
    ''' <remarks></remarks>
    Private Sub NABiSetHelpColor(peLabelHelpColor As ENUM_COMMONCOLOR)


        If peLabelHelpColor <> ENUM_COMMONCOLOR.NONE Then
            Me.NABiUseAutoSize = ENUM_LABELAUTOSIZE.NONE
            Me.Appearance.BackGradientStyle = GradientStyle.Vertical
            Me.BorderStyleOuter = UIElementBorderStyle.Rounded1
            Me.Size = New Size(10, 10)

        End If

        ''최한표20150702 - designmode 에서는 그냥 빠져나가는 것으로 한다...
        'If DesignMode Then
        '    Return
        'End If
        '최한표20150708 - NABiSetCommonColor 에서는 DB를 읽지 않는 것으로 보이므로, 빠져나가지 않고 진행하는 것으로 수정한다.

        moUIControlsCommon.NABiSetCommonColor()


        Select Case peLabelHelpColor
            Case ENUM_COMMONCOLOR.OUTPATIENT
                Me.Appearance.BackColor = moUIControlsCommon.NABiOutPatientBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiOutPatientBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiOutPatientBackColor

            Case ENUM_COMMONCOLOR.INPATIENT
                Me.Appearance.BackColor = moUIControlsCommon.NABiInPatientBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiInPatientBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiInPatientBackColor

            Case ENUM_COMMONCOLOR.DEREGISTER
                Me.Appearance.BackColor = moUIControlsCommon.NABiDeregisterBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiDeregisterBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiDeregisterBackColor

            Case ENUM_COMMONCOLOR.RESERVATION   '예약
                Me.Appearance.BackColor = moUIControlsCommon.NABiReservationBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiReservationBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiReservationBackColor

            Case ENUM_COMMONCOLOR.SECOND_INSURANCE  '이중보험
                Me.Appearance.BackColor = moUIControlsCommon.NABiSecondInsuranceBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiSecondInsuranceBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiSecondInsuranceBackColor

            Case ENUM_COMMONCOLOR.THIRD_INSURANCE   '삼중보험
                Me.Appearance.BackColor = moUIControlsCommon.NABiThirdInsuranceBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiThirdInsuranceBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiThirdInsuranceBackColor

            Case ENUM_COMMONCOLOR.DISCHARGE_ORDER   '퇴원지시
                Me.Appearance.BackColor = moUIControlsCommon.NABiDischargeOrderBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiDischargeOrderBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiDischargeOrderBackColor

            Case ENUM_COMMONCOLOR.DISCHARGE_SIMSA_REQ '퇴원심사요청
                Me.Appearance.BackColor = moUIControlsCommon.NABiDischargeSIMSAOrderBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiDischargeSIMSAOrderBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiDischargeSIMSAOrderBackColor

            Case ENUM_COMMONCOLOR.DISCHARGE_SIMSA_END   '퇴원심사완료
                Me.Appearance.BackColor = moUIControlsCommon.NABiDischargeSIMSACompleteBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiDischargeSIMSACompleteBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiDischargeSIMSACompleteBackColor

            Case ENUM_COMMONCOLOR.DISCHARGE '퇴원처리
                Me.Appearance.BackColor = moUIControlsCommon.NABiDischargeBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiDischargeBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiDischargeBackColor

            Case ENUM_COMMONCOLOR.FAKE_DISCHARGE    '가퇴원처리
                Me.Appearance.BackColor = moUIControlsCommon.NABiFakeDischargeBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiFakeDischargeBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiFakeDischargeBackColor

            Case ENUM_COMMONCOLOR.FAKE_DISCHARGE_SIMSA_END  '가퇴원심사완료
                Me.Appearance.BackColor = moUIControlsCommon.NABiFakeDischargeSIMSACompleteBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiFakeDischargeSIMSACompleteBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiFakeDischargeSIMSACompleteBackColor

            Case ENUM_COMMONCOLOR.GROUP     '묶음코드
                Me.Appearance.BackColor = moUIControlsCommon.NABiGroupCodeBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiGroupCodeBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiGroupCodeBackColor

            Case ENUM_COMMONCOLOR.EXPIRATION    '기간만료
                Me.Appearance.BackColor = moUIControlsCommon.NABiExpirationBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiExpirationBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiExpirationBackColor

            Case ENUM_COMMONCOLOR.INSURANCE_HUNDRED      '본인100%
                Me.Appearance.BackColor = moUIControlsCommon.NABiInsuranceHundredBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiInsuranceHundredBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiInsuranceHundredBackColor

            Case ENUM_COMMONCOLOR.NOGUB                  '비급여
                Me.Appearance.BackColor = moUIControlsCommon.NABiNoGubBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiNoGubBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiNoGubBackColor

            Case ENUM_COMMONCOLOR.HUNDRED_HUNDRED        '100/100
                Me.Appearance.BackColor = moUIControlsCommon.NABiHundredHundredBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiHundredHundredBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiHundredHundredBackColor
                
            Case ENUM_COMMONCOLOR.ICD                    '상병
                Me.Appearance.BackColor = moUIControlsCommon.NABiICDBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiICDBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiICDBackColor

            Case ENUM_COMMONCOLOR.GUB           '급여
                Me.Appearance.BackColor = moUIControlsCommon.NABiGubBackColor
                Me.Appearance.BackColor2 = moUIControlsCommon.NABiGubBackColor2
                Me.Appearance.BorderColor = moUIControlsCommon.NABiGubBackColor

            Case ENUM_COMMONCOLOR.NONE

        End Select


    End Sub

#End Region

#Region " Events "


    ''' <summary>
    ''' Label 사이즈 변경에 따른 FontSize 자동 설정
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' 2015-06-23 민유진 Created
    ''' </remarks>
    Protected Overrides Sub OnSizeChanged(e As EventArgs)

        If meAutoFontSize = ENUM_LABELAUTOSIZE.FONTAUTOSIZE Then

            Dim schangedText As String = Me.Text
            Dim oGraphic As Graphics = Me.CreateGraphics()
            Dim oinitFont As New Font(Me.Font.FontFamily, Me.Font.Size)

            NABiUseFontStyle = ENUM_TRUEFALSE.FALSE

            Using oGraphic
                Dim ocurrentSize As SizeF = oGraphic.MeasureString(schangedText, Me.Font)
                Dim sngfontSize As Single = Me.Font.Size
                Dim sngadaptSize As Single = 0

                If ocurrentSize.Width > Me.Width - 10 OrElse ocurrentSize.Height > Me.Height - 10 Then
                    While True
                        If sngfontSize < 5 Then
                            sngadaptSize = sngfontSize
                            Exit While
                        End If
                        sngfontSize -= 1
                        Dim ocheckFont As New Font(Me.Font.FontFamily, sngfontSize)
                        Using ocheckFont
                            Dim ocheckSize As SizeF = oGraphic.MeasureString(schangedText, ocheckFont)
                            If ocheckSize.Width < Me.Width - 10 AndAlso ocheckSize.Height < Me.Height - 10 Then
                                sngadaptSize = ocheckFont.Size
                                Exit While
                            End If
                        End Using
                    End While
                Else
                    Dim oinitSize As SizeF = oGraphic.MeasureString(schangedText, oinitFont)
                    'If oinitSize.Width > olabel.Width Then
                    If oinitSize.Width < Me.Width - 10 AndAlso oinitSize.Height < Me.Height - 10 Then
                        While True
                            sngfontSize += 1
                            Dim ocheckFont As New Font(Me.Font.FontFamily, sngfontSize)
                            Using ocheckFont
                                Dim ocheckSize As SizeF = oGraphic.MeasureString(schangedText, ocheckFont)
                                If ocheckSize.Width > Me.Width - 10 OrElse ocheckSize.Height > Me.Height - 10 Then
                                    sngadaptSize = ocheckFont.Size
                                    Exit While
                                End If
                            End Using
                        End While
                    End If
                End If

                If sngadaptSize <> 0 Then
                    Me.Font = New Font(Me.Font.FontFamily, sngadaptSize)
                    oinitFont.Dispose()
                Else
                    Me.Font = oinitFont
                End If

            End Using

        End If

        MyBase.OnSizeChanged(e)

    End Sub

#End Region

End Class

''' <summary>
''' 움직이는 Label
''' </summary>
''' <remarks>
''' 2015-06-18 민유진 Created
''' </remarks>
Public Class NABiLabelDrawFilter
    Implements IUIElementDrawFilter
    Implements iNABiControlProperty


#Region " Private Fields "

    Private oNabiLabel As UltraLabel
    Private oNabiText As StringBuilder
    Private oTimer As New Timer()

#End Region

#Region " Properties "

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseFontStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseFontStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseNABiStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseNABiStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataProtect As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiDataProtect

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataType As ENUM_PROTECTDATA Implements iNABiControlProperty.NABiDataType

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindDTL As ENUM_BINDDETAIL Implements iNABiControlProperty.NABiBindDTL

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindMaster As ENUM_BINDMASTER Implements iNABiControlProperty.NABiBindMaster

#End Region

#Region " Constructors "

    Public Sub New(ByVal pLabel As UltraLabel)

        Me.oNabiLabel = pLabel
        Me.oNabiText = New StringBuilder(pLabel.Text)
        Me.oTimer.Interval = 200
        AddHandler Me.oTimer.Tick, AddressOf timer_Tick

        Me.oNabiLabel.UseHotTracking = DefaultableBoolean.True
        oTimer.Start()

        AddHandler Me.oNabiLabel.TextChanged, AddressOf NabiLabel_TextChanged
        AddHandler Me.oNabiLabel.MouseHover, AddressOf NabiLabel_MouseOver
        AddHandler Me.oNabiLabel.MouseLeave, AddressOf NabiLabel_MouseLeave

    End Sub

#End Region

#Region " Public Methods "

    Public Function NABiDrawElement(drawPhase As DrawPhase, ByRef drawParams As UIElementDrawParams) As Boolean Implements IUIElementDrawFilter.DrawElement

        Dim oAlign As HAlign = drawParams.AppearanceData.TextHAlign
        Dim sfrmtStr As New StringFormat()

        If oAlign = HAlign.Left Then
            sfrmtStr.Alignment = StringAlignment.Near
        Else
            If oAlign = HAlign.Center Then
                sfrmtStr.Alignment = StringAlignment.Center
            Else
                If oAlign = HAlign.Right Then
                    sfrmtStr.Alignment = StringAlignment.Far
                End If
            End If
        End If

        Dim rectF As New RectangleF(drawParams.Element.RectInsideBorders.Left, drawParams.Element.RectInsideBorders.Top, drawParams.Element.RectInsideBorders.Width, drawParams.Element.RectInsideBorders.Height)
        drawParams.Graphics.DrawString(Me.oNabiText.ToString(), drawParams.Font, drawParams.TextBrush, rectF, sfrmtStr)

        Me.NABiUpdateText()

        Return True

    End Function

    Public Function GetPhasesToFilter(ByRef drawParams As UIElementDrawParams) As DrawPhase Implements IUIElementDrawFilter.GetPhasesToFilter

        If TypeOf drawParams.Element Is TextUIElementBase Then
            Return DrawPhase.BeforeDrawForeground
        End If
        Return DrawPhase.None

    End Function

    Public Sub NABiSetFontStyle(peFontStyle As ENUM_TRUEFALSE) Implements iNABiControlProperty.NABiSetFontStyle
        'Nothing
    End Sub

#End Region

#Region " Private Methods "

    Private Sub NABiUpdateText()

        Dim chText As Char

        If Me.oNabiText.Length = 0 Then
            Return
        End If

        chText = Me.oNabiText(0)
        Me.oNabiText.Remove(0, 1)
        Me.oNabiText.Append(chText)

    End Sub

#End Region

#Region " Events "

    Private Sub timer_Tick(sender As Object, e As EventArgs)
        Me.oNabiLabel.Refresh()
    End Sub

    Private Sub NabiLabel_TextChanged(sender As Object, e As EventArgs)
        Me.oNabiText = New StringBuilder(Me.oNabiLabel.Text)
    End Sub

    Private Sub NabiLabel_MouseOver(sender As Object, e As EventArgs)
        oNabiLabel.DrawFilter = Nothing
        Me.oTimer.Stop()
    End Sub

    Private Sub NabiLabel_MouseLeave(sender As Object, e As EventArgs)
        oNabiLabel.DrawFilter = Me
        oTimer.Start()
    End Sub

#End Region

End Class


'Module Extensions

'    ''' <summary>
'    ''' Extension method to return if the control is in design mode
'    ''' </summary>
'    ''' <param name="control">Control to examine</param>
'    ''' <returns>True if in design mode, otherwise false</returns>
'    <System.Runtime.CompilerServices.Extension> _
'    Public Function IsInDesignMode(control As System.Windows.Forms.Control) As Boolean
'        Return ResolveDesignMode(control)
'    End Function


'    ''' <summary>
'    ''' Method to test if the control or it's parent is in design mode
'    ''' </summary>
'    ''' <param name="control">Control to examine</param>
'    ''' <returns>True if in design mode, otherwise false</returns>
'    Private Function ResolveDesignMode(control As System.Windows.Forms.Control) As Boolean
'        Dim designModeProperty As System.Reflection.PropertyInfo
'        Dim designMode As Boolean


'        ' Get the protected property
'        designModeProperty = control.[GetType]().GetProperty("DesignMode", System.Reflection.BindingFlags.Instance Or System.Reflection.BindingFlags.NonPublic)


'        ' Get the controls DesignMode value
'        designMode = CBool(designModeProperty.GetValue(control, Nothing))


'        ' Test the parent if it exists
'        If control.Parent IsNot Nothing Then
'            designMode = designMode Or ResolveDesignMode(control.Parent)
'        End If


'        Return designMode
'    End Function
'End Module
