﻿Imports System.ComponentModel
Imports Infragistics.Win.UltraWinStatusBar

Public Class NABiStatusBar
    Inherits Infragistics.Win.UltraWinStatusBar.UltraStatusBar
    Implements iNABiControlProperty
    Implements ISupportInitialize

#Region " Private Fields "

    Private meUseNABiStyle As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE
    Private meUseFontStyle As ENUM_TRUEFALSE = ENUM_TRUEFALSE.NONE

    ' *** Control모두 Initalizing 되었는지 여부
    Private mbIsInitializing As Boolean = False

#End Region

#Region " Properties "

    <CategoryAttribute("NABi Design"), Browsable(True), Description("Font style 설정 (맑은고딕/9pt)")>
    Public Property NABiUseFontStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseFontStyle
        Get
            Return meUseFontStyle
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meUseFontStyle = Value
            NABiSetFontStyle(meUseFontStyle)
            Me.Invalidate()
        End Set
    End Property

    <CategoryAttribute("NABi Design"), Browsable(True), Description("NABi style 설정")>
    Public Property NABiUseNABiStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseNABiStyle
        Get
            Return meUseNABiStyle
        End Get
        Set(ByVal Value As ENUM_TRUEFALSE)
            meUseNABiStyle = Value
            If (Not Me.mbIsInitializing) Then
                If meUseNABiStyle = ENUM_TRUEFALSE.TRUE Then
                    NABiSetStatusStyle()
                Else
                    'NABiClearRibbonStyle()
                End If
            End If
            Me.Invalidate()
        End Set

    End Property

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindDTL As ENUM_BINDDETAIL Implements iNABiControlProperty.NABiBindDTL

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindMaster As ENUM_BINDMASTER Implements iNABiControlProperty.NABiBindMaster

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataProtect As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiDataProtect

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataType As ENUM_PROTECTDATA Implements iNABiControlProperty.NABiDataType

#End Region

#Region " Public Methods "
    ''' <summary>
    ''' Control이 올라오기 전
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub BeginInit()

        Me.mbIsInitializing = True

    End Sub

    Public Sub NABiSetFontStyle(peFontStyle As ENUM_TRUEFALSE) Implements iNABiControlProperty.NABiSetFontStyle
        If peFontStyle = ENUM_TRUEFALSE.TRUE Then

            Me.Appearance.FontData.Name = NABI_FONTNAME
            Me.Appearance.FontData.SizeInPoints = NABI_FONTSIZE

        End If

    End Sub

#End Region

#Region " Private Methods "

    ''' <summary>
    ''' 컨트롤이 올라온 후 설정
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub EndInit()
        Me.mbIsInitializing = False

        If Me.meUseNABiStyle = ENUM_TRUEFALSE.TRUE Then
            NABiSetStatusStyle()
        End If
    End Sub

    ''' <summary>
    ''' NABi Style 적용
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub NABiSetStatusStyle()

        '처음 로드시에만 적용
        If meUseNABiStyle = ENUM_TRUEFALSE.TRUE AndAlso DesignMode Then
            Dim oSttPanel As UltraStatusPanel

            '사용자 정보
            oSttPanel = Me.Panels.Add("NABiUser", PanelStyle.Text)
            oSttPanel.Text = "NABiUser"
            oSttPanel.Width = 150
            oSttPanel.SizingMode = PanelSizingMode.Adjustable

            '작업일자
            oSttPanel = Me.Panels.Add("NABiWorkDate", PanelStyle.Text)
            oSttPanel.SizingMode = PanelSizingMode.Adjustable
            oSttPanel.Width = 120
            oSttPanel.Text = "NABiWorkDate"

            '프로그래스바
            oSttPanel = Me.Panels.Add("NABiProgress", PanelStyle.Progress)
            oSttPanel.Width = 150
            oSttPanel.SizingMode = PanelSizingMode.Adjustable
            With oSttPanel.ProgressBarInfo

                .Appearance.BackColor = Color.Transparent
                .Appearance.BackColor2 = Color.Transparent
                .Appearance.ImageBackground = My.Resources.progress_bg
                .FillAppearance.BackColor = Color.Transparent
                .FillAppearance.BackColor2 = Color.Transparent
                .FillAppearance.ImageBackground = My.Resources.progress_bar
                .Minimum = 0
                .Maximum = 100
                .ShowLabel = False
                .Style = Infragistics.Win.UltraWinProgressBar.ProgressBarStyle.Continuous

            End With

            '메세지 정보
            oSttPanel = Me.Panels.Add("NABiMessage", PanelStyle.Text)
            oSttPanel.SizingMode = PanelSizingMode.Spring
            oSttPanel.Text = "NABiMessage"

            NABiSetStatusDesign()

        End If

    End Sub

    ''' <summary>
    ''' Statusbar design
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub NABiSetStatusDesign()

        Me.UseOsThemes = Infragistics.Win.DefaultableBoolean.False
        Me.Appearance.BackColor = Color.FromArgb(225, 225, 225)
        Me.Appearance.BackColor2 = Color.FromArgb(194, 194, 194)
        Me.Appearance.BackGradientStyle = Infragistics.Win.GradientStyle.Vertical
        Me.Appearance.BorderColor = Color.FromArgb(194, 194, 194)

        Me.BorderStyle = Infragistics.Win.UIElementBorderStyle.None
        Me.BorderStylePanel = Infragistics.Win.UIElementBorderStyle.Rounded1

        Me.Size = New Size(Me.Size.Width, 28)

    End Sub

#End Region

End Class
