﻿Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Windows.Forms.Design
Imports System.Drawing.Drawing2D
Imports System.Runtime.InteropServices


<ComVisible(False), Designer(GetType(NABiTransparentControlDesigner)), _
ClassInterface(ClassInterfaceType.AutoDispatch), _
DefaultProperty("Text"), DefaultEvent("Click")> _
Public Class NABiTransparentControl
    Inherits Control
    Implements iNABiControlProperty

#Region " Private Fields "

    ''' <summary>
    ''' Event Invoked when Property Changed
    ''' </summary>
    ''' <param name="pRecreateHandle"></param>
    ''' <remarks></remarks>
    Protected Event Event_PropertyChanged(ByVal pRecreateHandle As Boolean)

    Private moTextAlign As ContentAlignment = ContentAlignment.MiddleCenter
    Private moImage As Image

#End Region

#Region " Properties "

    <CategoryAttribute("NABi Design"), Browsable(True), Description("Gets or sets the specified image / logo for the control.")> _
    Public Property NABiImage() As Image
        Get
            Return moImage
        End Get
        Set(ByVal value As Image)
            moImage = value
            RaiseEvent Event_PropertyChanged(True)
        End Set
    End Property


    <Browsable(True), Description("Gets or sets the text inside the control")> _
    Public Overrides Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
            RaiseEvent Event_PropertyChanged(True)
        End Set
    End Property

    ''' <summary>
    '''  Specifies alignment of <see cref="Text"></see> on the control.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <CategoryAttribute("NABi Design"), Browsable(True), Description("Gets or sets the alignment of text inside the control")> _
    Public Property NABiTextAlign() As ContentAlignment
        Get
            Return moTextAlign
        End Get
        Set(ByVal value As ContentAlignment)
            moTextAlign = value
            RaiseEvent Event_PropertyChanged(True)
        End Set
    End Property

    ''' <summary>
    ''' Summarize the information needed when creating a control.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Protected Overrides ReadOnly Property CreateParams() As CreateParams
        Get
            Dim cp As CreateParams = MyBase.CreateParams
            cp.ExStyle = cp.ExStyle Or &H20  ' Turn on WS_EX_TRANSPARENT
            Return cp

        End Get
    End Property

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseFontStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseFontStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiUseNABiStyle As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiUseNABiStyle

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataProtect As ENUM_TRUEFALSE Implements iNABiControlProperty.NABiDataProtect

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiDataType As ENUM_PROTECTDATA Implements iNABiControlProperty.NABiDataType

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindDTL As ENUM_BINDDETAIL Implements iNABiControlProperty.NABiBindDTL

    <CategoryAttribute("NABi Design"), Browsable(False), Description("숨김 속성")>
    Public Property NABiBindMaster As ENUM_BINDMASTER Implements iNABiControlProperty.NABiBindMaster

#End Region

#Region " Constructors "

    Public Sub New()

        Me.SetStyle(ControlStyles.SupportsTransparentBackColor, True)
        Me.SetStyle(ControlStyles.ResizeRedraw, True)
        Me.SetStyle(ControlStyles.Opaque, False)
        Me.SetStyle(ControlStyles.OptimizedDoubleBuffer, False)

        AddHandler Me.Event_PropertyChanged, AddressOf OnPropertyChanged
        AddHandler Me.MouseHover, AddressOf OnRefreshed
        RaiseEvent Event_PropertyChanged(False)
    End Sub

#End Region

#Region " Events "

    ''' <summary>
    ''' Invoked when Property Changed
    ''' </summary>
    ''' <param name="pRecreateHandle"></param>
    ''' <remarks></remarks>
    Private Sub OnPropertyChanged(ByVal pRecreateHandle As Boolean)
        If (Not Me.Parent Is Nothing) Then
            'Dim _ControlRect As Rectangle = New Rectangle(Me.Location, Me.Size)
            'Me.Parent.Invalidate(_ControlRect, True)
        End If
        If (pRecreateHandle = True) Then Me.RecreateHandle()
        Me.Invalidate()
    End Sub

    Private Sub OnRefreshed(sender As Object, e As EventArgs)
        OnPropertyChanged(True)
    End Sub

#End Region

#Region " Protected Events "

    ''' <summary>
    ''' Call(s) to Paints the control background.
    ''' </summary>
    ''' <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
    ''' <remarks><b>Here don't allow the background to be painted</b>, <i>Otherwise the background draw task spoil the recently repainted parent control content by crushing out the OnPaintBackground() method </i></remarks>
    Protected Overrides Sub OnPaintBackground(ByVal e As PaintEventArgs)
        ' NOTHING TODO:
    End Sub


    ''' <summary>
    ''' Raises event.
    ''' </summary>
    ''' <param name="e">An System.EventArgs that contains the event data.</param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnFontChanged(ByVal e As System.EventArgs)
        RaiseEvent Event_PropertyChanged(True)
    End Sub

    ''' <summary>
    ''' Raises ForeColorChanged event.
    ''' </summary>
    ''' <param name="e">An System.EventArgs that contains the event data.</param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnForeColorChanged(ByVal e As System.EventArgs)
        RaiseEvent Event_PropertyChanged(True)
    End Sub

    ''' <summary>
    ''' Raises the System.Windows.Forms.Control.Paint event
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        e.Graphics.CompositingMode = CompositingMode.SourceOver
        e.Graphics.CompositingQuality = CompositingQuality.GammaCorrected
        e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

        Call NABiDrawControlText(e.Graphics, Me.ClientRectangle)
        Call NABiDrawControlImage(e.Graphics, Me.ClientRectangle)
    End Sub

#End Region

#Region " Protected Methods "

    ''' <summary>
    ''' Call this from OnPaint() to Draw Control's Text
    ''' </summary>
    ''' <param name="g">Graphics Object Used to Paint</param>
    ''' <remarks></remarks>
    Protected Sub NABiDrawControlText(ByRef g As Graphics, ByVal pClientRect As Rectangle)
        If (Not IsNothing(moImage)) Then pClientRect.Width = pClientRect.Width - 16
        Dim oTextRect As Rectangle = NABiGetTextRectangle(Me.Text, Me.Font, g, pClientRect, Me.NABiTextAlign)
        g.DrawString(Me.Text, Me.Font, New SolidBrush(Me.ForeColor), oTextRect)
    End Sub

    ''' <summary>
    ''' Call this from OnPaint() to Draw Control's Image
    ''' </summary>
    ''' <param name="g">Graphics Object Used to Paint</param>
    ''' <param name="_ClientRect">The Rectangle that represents the client area of the control.</param>
    ''' <remarks></remarks>
    Protected Sub NABiDrawControlImage(ByRef g As Graphics, ByVal _ClientRect As Rectangle)
        Dim pPoint As Point

        If (Not IsNothing(moImage)) Then
            pPoint = New Point(_ClientRect.Left, _ClientRect.Top)
            'g.DrawImage(m_Image, (_ClientRect.Width - 18), CInt(Fix((_ClientRect.Height / 2) - 8)), m_ImageSize, m_ImageSize)
            g.DrawImage(moImage, pPoint)
        End If
    End Sub

    ''' <summary>
    ''' Text를 그릴 사각영역
    ''' </summary>
    ''' <param name="pText"></param>
    ''' <param name="pFont"></param>
    ''' <param name="g"></param>
    ''' <param name="pClientRect"></param>
    ''' <param name="pTextAlign"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Protected Function NABiGetTextRectangle(ByVal pText As String, ByVal pFont As Font, ByRef g As Graphics, _
                                    ByVal pClientRect As Rectangle, ByVal pTextAlign As ContentAlignment) As Rectangle
        Dim oTextSize As SizeF = g.MeasureString(pText, pFont)
        Dim iTextLeft As Integer
        oTextSize.Width = oTextSize.Width + 4

        Select Case pTextAlign
            Case ContentAlignment.TopLeft
                Return pClientRect

            Case ContentAlignment.TopCenter
                iTextLeft = (pClientRect.Width / 2) - (CInt(Fix(oTextSize.Width)) / 2)

                Return New Rectangle((pClientRect.Left + iTextLeft), pClientRect.Top, CInt(Fix(oTextSize.Width)), _
                                   CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.TopRight
                iTextLeft = (pClientRect.Width - oTextSize.Width)

                Return New Rectangle((pClientRect.Left + iTextLeft), pClientRect.Top, CInt(Fix(oTextSize.Width)), _
                                   CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.MiddleLeft
                Return New Rectangle(pClientRect.Left, CInt(Fix(pClientRect.Height / 2)) - CInt(Fix(oTextSize.Height / 2)), _
                                   CInt(Fix(oTextSize.Width)), CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.MiddleCenter
                iTextLeft = (pClientRect.Width / 2) - (CInt(Fix(oTextSize.Width)) / 2)

                Return New Rectangle((pClientRect.Left + iTextLeft), _
                                   CInt(Fix(pClientRect.Height / 2)) - CInt(Fix(oTextSize.Height / 2)), _
                                   CInt(Fix(oTextSize.Width)), CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.MiddleRight
                iTextLeft = (pClientRect.Width - oTextSize.Width)

                Return New Rectangle((pClientRect.Left + iTextLeft), _
                                   CInt(Fix(pClientRect.Height / 2)) - CInt(Fix(oTextSize.Height / 2)), _
                                   CInt(Fix(oTextSize.Width)), CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.BottomLeft
                Return New Rectangle(pClientRect.Left, CInt(Fix(pClientRect.Bottom - oTextSize.Height)), _
                                   CInt(Fix(oTextSize.Width)), CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.BottomCenter
                iTextLeft = (pClientRect.Width / 2) - (CInt(Fix(oTextSize.Width)) / 2)

                Return New Rectangle((pClientRect.Left + iTextLeft), CInt(Fix(pClientRect.Bottom - oTextSize.Height)), _
                                   CInt(Fix(oTextSize.Width)), CInt(Fix(oTextSize.Height)))

            Case ContentAlignment.BottomRight
                iTextLeft = (pClientRect.Width - oTextSize.Width)

                Return New Rectangle((pClientRect.Left + iTextLeft), CInt(Fix(pClientRect.Bottom - oTextSize.Height)), _
                                   CInt(Fix(oTextSize.Width)), CInt(Fix(oTextSize.Height)))
        End Select

    End Function

#End Region

#Region " Public Methods "

    Public Sub NABiSetFontStyle(peFontStyle As ENUM_TRUEFALSE) Implements iNABiControlProperty.NABiSetFontStyle
        'Nothing
    End Sub

#End Region

End Class



Public Class NABiTransparentControlDesigner
    Inherits System.Windows.Forms.Design.ControlDesigner

    ''' <summary>
    ''' Designer 제외 프로퍼티리스트
    ''' </summary>
    ''' <param name="pProperties"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub PostFilterProperties(ByVal pProperties As System.Collections.IDictionary)

        pProperties.Remove("BackColor")
        pProperties.Remove("BackgroundImage")
        pProperties.Remove("BackgroundImageLayout")
        pProperties.Remove("RightToLeft")
        pProperties.Remove("TabStop")
        pProperties.Remove("TabIndex")
        pProperties.Remove("AutoSize")

        MyBase.PostFilterProperties(pProperties)

    End Sub

End Class

