﻿Imports System
Imports System.ComponentModel
Imports System.ComponentModel.Design.Serialization
Imports System.Reflection
Imports System.Globalization
Imports System.Runtime.Serialization
Imports System.Security.Permissions

Imports Infragistics.Shared
Imports Infragistics.Shared.Serialization
Imports Infragistics.Win
Imports Infragistics.Win.UltraWinToolbars

Namespace ToolProvider

    'The following sample classes demonstrate how to create custom tool types that will work in
    'UltraToolbarsManager.  Refer to the comments in the ToolProvider.cs file for information on 
    'how to expose your custom tools in the UltraToolbarsManager designtime Customizer, allowing
    'developers to select and create instances of your custom tools when desiging their toolbars
    'menus.
    '
    'In order for custom tool types to function properly in UltraToolbarManager they must be derived
    'from one of UltraToolbarManager's built-in tool types.  The built-in tool types are:
    '
    '		ToolBase					- the abstract base class from which all other tools are derived.
    '		ButtonTool					- a button tool
    '		ComboBoxTool				- a combobox tool
    '		ControlContainerTool		- a tool that can serve as a container for any .NET WinForms control
    '		FontListTool				- a tool that displays a list of the fonts available on the system in a dropdown list.
    '		LabelTool					- a tool that displays text
    '		ListTool					- a tool that can bve placed on a menu to display a list of items.
    '		MDIWindowListTool			- a tool that can be placed on a menu to display a list of the currently open MDI child windows
    '		PopupColorListTool			- a tool that displays a popup color picker
    '		PopupControlContainerTool	- a tool that can display eny .NET WinForms control in a popup window.
    '		PopupMenuTool				- a tool that displays a popup menu
    '		StateButtonTool				- a state button tool
    '		TextboxTool					- a textbox tool
    '
    'Each sample class (CustomButtonTool and CustomComboBoxTool) contains a single custom property called CustomProp.
    'The property is of type string and is included to show how you would add support for custom properties in your
    'derived tool classes.  This includes code to demonstrate how to share the property setting across multiple
    'instances of a single root tool based on your custom tool type and how to load/persist your tool's custom property
    'settings when the control's layout is loaded/saved.
    '
    'The ExtraSharedProps helper class defined for each tool type is a part of the UltraToolbarManager's tool
    'architecture that allows tool property settings to be shared across all instances of a single root tool
    'just as the SharedProps object does for each tool with the built-in shared tool properties.  The ExtraSharedProps
    'helper class that you create for your derived tool will, in effect, shadow the custom properties of your tool.  Property
    'gets/sets for each custom property on your custom tool will delegate processing to similary named properties
    'on the tool's ExtraSharedProps object.   Regardless of how many instances of a root version of your custom tool
    'are created by the developer, there will be a single instance of your tool's ExtraSharedProps object.
    '
    'In order for your custom tools to persist correctly at designtime it must use a custom type converter.  Be sure to
    'include the TypeConverter attribute on your class and create an ExpandableObjectConverter derived TypeConverter class 
    'as show in the samples.  Simply use the type convert code provided in the samples and change the reference tool type
    'to your tool's type.
    '
    'The sample project also contains a PropertyIds.cs file that defines enuemrations for all custom proeprties.
    'These property id enumerations are used when firing property change notifications for your tool's custom 
    'properties.  The PropertyIds.cs included with the sample starts numbering the enumerations starting at 5000 to 
    'avoid conflicts with the ids assigned to the proeprties of the built-in tools (built-in tool property ids start
    'at 0).  As you add properties to your custom tool, be sure to add property ids as well.  

    Public Interface INABiCustomProperty
        Property NABiGroupKeyProp() As String
        Property NABiGroupNameProp() As String
        Property NABiTabIndexProp() As ENUM_TAB_LIST
        Property NABiButtonTypeProp() As ENUM_BUTTONTYPE
    End Interface

#Region " CustomButtonTool Class "
    'CustomButtonTool Class
    <TypeConverter(GetType(CustomButtonTool.CustomButtonToolConverter)), _
    Serializable()> _
    Public Class CustomButtonTool
        Inherits ButtonTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomButtonTool.CustomButtonToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                    Case "NABiButtonTypeProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_ButtonType))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0
        Friend Shared ReadOnly DEFAULTVALUE_NABIBUTTONTYPEPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomButtonTool = New CustomButtonTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
                Utils.SerializeProperty(info, "NABiButtonTypeProp", Me.NABiButtonTypeProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex
                Me.ExtraSharedProps.m_ButtonType = Me.SerializedProps.m_ButtonType

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "

        '2016-01-21 이재영 추가 : Button Type Property
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp
            Get
                Return Me.ExtraSharedProps.m_ButtonType
            End Get
            Set(value As ENUM_BUTTONTYPE)
                Me.ExtraSharedProps.m_ButtonType = value
            End Set
        End Property

        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property
        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomButtonTool.CustomButtonToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomButtonTool.CustomButtonToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomButtonTool Type Converter "
        'Type converter for the CustomButtonTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomButtonToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomButtonTool).GetConstructor(New Type() {GetType(String)})
                    Dim buttonTool As CustomButtonTool = CType(value, CustomButtonTool)

                    Return New InstanceDescriptor(ci, New Object() {buttonTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomButtonTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomButtonToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomButtonToolExtraSharedProps = New CustomButtonToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomButtonToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomButtonToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomButtonToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomButtonTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomButtonTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomButtonTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomButtonTool.DEFAULTVALUE_NABITABINDEXPROP
            '2016-01-21 이재영 : Button type을 받아오기 위한 변수
            Friend m_ButtonType As Integer = ENUM_BUTTONTYPE.NONE

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomButtonTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

            '2016-01-21 이재영 추가
            Friend Property NABiButtonTypeProp() As ENUM_BUTTONTYPE
                Get
                    Return Me.m_ButtonType
                End Get
                Set(value As ENUM_BUTTONTYPE)
                    If (value <> Me.m_ButtonType) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiButtonTypeProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties

    End Class

#End Region  'CustomButtonTool Class

#Region " CustomComboBoxTool Class "
    'CustomComboBoxTool Class
    <TypeConverter(GetType(CustomComboBoxTool.CustomComboBoxToolConverter)), _
    Serializable()> _
    Public Class CustomComboBoxTool
        Inherits ComboBoxTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomComboBoxTool.CustomComboBoxToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomComboBoxTool = New CustomComboBoxTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        'This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)
        End Function

#End Region  'CustomProp

#End Region 'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomComboBoxTool.CustomComboBoxToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomComboBoxTool.CustomComboBoxToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region  'SerializedProps

#End Region 'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomComboBoxTool Type Converter "
        'Type converter for the CustomComboBoxTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomComboBoxToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomComboBoxTool).GetConstructor(New Type() {GetType(String)})
                    Dim ComboBoxTool As CustomComboBoxTool = CType(value, CustomComboBoxTool)

                    Return New InstanceDescriptor(ci, New Object() {ComboBoxTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomComboBoxTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region 'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomComboBoxToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomComboBoxToolExtraSharedProps = New CustomComboBoxToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomComboBoxToolExtraSharedProps)
            End Get
        End Property

#End Region 'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomComboBoxToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomComboBoxToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomComboBoxTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomButtonTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomButtonTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomButtonTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region 'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomComboBoxTool)
                'Initialize member variables.
                Me.owningTool = owningTool

            End Sub

#End Region 'Constructor

#Region " Properties "

#Region " CustomProp "
            'This property supports the same-named property on our owning tool.
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property


#End Region  'CustomProp

#End Region 'Properties
        End Class

#End Region 'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties

    End Class

#End Region  'CustomComboBoxTool Class

#Region " CustomControlContainerTool Class "
    'CustomControlContainerTool Class
    <TypeConverter(GetType(CustomControlContainerTool.CustomControlContainerToolConverter)), _
    Serializable()> _
    Public Class CustomControlContainerTool
        Inherits ControlContainerTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomControlContainerTool.CustomControlContainerToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomControlContainerTool = New CustomControlContainerTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomControlContainerTool.CustomControlContainerToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomControlContainerTool.CustomControlContainerToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomControlContainerTool Type Converter "
        'Type converter for the CustomControlContainerTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomControlContainerToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomControlContainerTool).GetConstructor(New Type() {GetType(String)})
                    Dim ControlContainerTool As CustomControlContainerTool = CType(value, CustomControlContainerTool)

                    Return New InstanceDescriptor(ci, New Object() {ControlContainerTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomControlContainerTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomControlContainerToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomControlContainerToolExtraSharedProps = New CustomControlContainerToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomControlContainerToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomControlContainerToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomControlContainerToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomControlContainerTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomControlContainerTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomControlContainerTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomControlContainerTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomControlContainerTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomControlContainerTool Class

#Region " CustomFontListTool Class "
    'CustomFontListTool Class
    <TypeConverter(GetType(CustomFontListTool.CustomFontListToolConverter)), _
    Serializable()> _
    Public Class CustomFontListTool
        Inherits FontListTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomFontListTool.CustomFontListToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomFontListTool = New CustomFontListTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomFontListTool.CustomFontListToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomFontListTool.CustomFontListToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomFontListTool Type Converter "
        'Type converter for the CustomFontListTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomFontListToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomFontListTool).GetConstructor(New Type() {GetType(String)})
                    Dim FontListTool As CustomFontListTool = CType(value, CustomFontListTool)

                    Return New InstanceDescriptor(ci, New Object() {FontListTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomFontListTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomFontListToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomFontListToolExtraSharedProps = New CustomFontListToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomFontListToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomFontListToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomFontListToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomFontListTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomFontListTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomFontListTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomFontListTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomFontListTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomFontListTool Class

#Region " CustomLabelTool Class "
    'CustomLabelTool Class
    <TypeConverter(GetType(CustomLabelTool.CustomLabelToolConverter)), _
    Serializable()> _
    Public Class CustomLabelTool
        Inherits LabelTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomLabelTool.CustomLabelToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomLabelTool = New CustomLabelTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomLabelTool.CustomLabelToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomLabelTool.CustomLabelToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomLabelTool Type Converter "
        'Type converter for the CustomLabelTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomLabelToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomLabelTool).GetConstructor(New Type() {GetType(String)})
                    Dim LabelTool As CustomLabelTool = CType(value, CustomLabelTool)

                    Return New InstanceDescriptor(ci, New Object() {LabelTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomLabelTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomLabelToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomLabelToolExtraSharedProps = New CustomLabelToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomLabelToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomLabelToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomLabelToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomLabelTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomLabelTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomLabelTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomLabelTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomLabelTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomLabelTool Class

#Region " CustomListTool Class "
    'CustomListTool Class
    <TypeConverter(GetType(CustomListTool.CustomListToolConverter)), _
    Serializable()> _
    Public Class CustomListTool
        Inherits ListTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomListTool.CustomListToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomListTool = New CustomListTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomListTool.CustomListToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomListTool.CustomListToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomListTool Type Converter "
        'Type converter for the CustomListTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomListToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomListTool).GetConstructor(New Type() {GetType(String)})
                    Dim ListTool As CustomListTool = CType(value, CustomListTool)

                    Return New InstanceDescriptor(ci, New Object() {ListTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomListTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomListToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomListToolExtraSharedProps = New CustomListToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomListToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomListToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomListToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomListTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomListTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomListTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomListTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomListTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomListTool Class

#Region " CustomMDIWindowListTool Class "
    'CustomMDIWindowListTool Class
    <TypeConverter(GetType(CustomMDIWindowListTool.CustomMDIWindowListToolConverter)), _
    Serializable()> _
    Public Class CustomMDIWindowListTool
        Inherits MdiWindowListTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomMDIWindowListTool.CustomMDIWindowListToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomMDIWindowListTool = New CustomMDIWindowListTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomMDIWindowListTool.CustomMDIWindowListToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomMDIWindowListTool.CustomMDIWindowListToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomMDIWindowListTool Type Converter "
        'Type converter for the CustomMDIWindowListTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomMDIWindowListToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomMDIWindowListTool).GetConstructor(New Type() {GetType(String)})
                    Dim MDIWindowListTool As CustomMDIWindowListTool = CType(value, CustomMDIWindowListTool)

                    Return New InstanceDescriptor(ci, New Object() {MDIWindowListTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomMDIWindowListTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomMDIWindowListToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomMDIWindowListToolExtraSharedProps = New CustomMDIWindowListToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomMDIWindowListToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomMDIWindowListToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomMDIWindowListToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomMDIWindowListTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomMDIWindowListTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomMDIWindowListTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomMDIWindowListTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomMDIWindowListTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomMDIWindowListTool Class

#Region " CustomPopupControlContainerTool Class "
    'CustomPopupControlContainerTool Class
    <TypeConverter(GetType(CustomPopupControlContainerTool.CustomPopupControlContainerToolConverter)), _
    Serializable()> _
    Public Class CustomPopupControlContainerTool
        Inherits PopupControlContainerTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomPopupControlContainerTool.CustomPopupControlContainerToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomPopupControlContainerTool = New CustomPopupControlContainerTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomPopupControlContainerTool.CustomPopupControlContainerToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomPopupControlContainerTool.CustomPopupControlContainerToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomPopupControlContainerTool Type Converter "
        'Type converter for the CustomPopupControlContainerTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomPopupControlContainerToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomPopupControlContainerTool).GetConstructor(New Type() {GetType(String)})
                    Dim PopupControlContainerTool As CustomPopupControlContainerTool = CType(value, CustomPopupControlContainerTool)

                    Return New InstanceDescriptor(ci, New Object() {PopupControlContainerTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomPopupControlContainerTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomPopupControlContainerToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomPopupControlContainerToolExtraSharedProps = New CustomPopupControlContainerToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomPopupControlContainerToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomPopupControlContainerToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomPopupControlContainerToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomPopupControlContainerTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomPopupControlContainerTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomPopupControlContainerTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomPopupControlContainerTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomPopupControlContainerTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomPopupControlContainerTool Class

#Region " CustomPopupMenuTool Class "
    'CustomPopupMenuTool Class
    <TypeConverter(GetType(CustomPopupMenuTool.CustomPopupMenuToolConverter)), _
    Serializable()> _
    Public Class CustomPopupMenuTool
        Inherits PopupMenuTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomPopupMenuTool.CustomPopupMenuToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomPopupMenuTool = New CustomPopupMenuTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomPopupMenuTool.CustomPopupMenuToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomPopupMenuTool.CustomPopupMenuToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomPopupMenuTool Type Converter "
        'Type converter for the CustomPopupMenuTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomPopupMenuToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomPopupMenuTool).GetConstructor(New Type() {GetType(String)})
                    Dim PopupMenuTool As CustomPopupMenuTool = CType(value, CustomPopupMenuTool)

                    Return New InstanceDescriptor(ci, New Object() {PopupMenuTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomPopupMenuTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomPopupMenuToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomPopupMenuToolExtraSharedProps = New CustomPopupMenuToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomPopupMenuToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomPopupMenuToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomPopupMenuToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomPopupMenuTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomPopupMenuTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomPopupMenuTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomPopupMenuTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomPopupMenuTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomPopupMenuTool Class

#Region " CustomStateButtonTool Class "
    'CustomStateButtonTool Class
    <TypeConverter(GetType(CustomStateButtonTool.CustomStateButtonToolConverter)), _
    Serializable()> _
    Public Class CustomStateButtonTool
        Inherits StateButtonTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomStateButtonTool.CustomStateButtonToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomStateButtonTool = New CustomStateButtonTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomStateButtonTool.CustomStateButtonToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomStateButtonTool.CustomStateButtonToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomStateButtonTool Type Converter "
        'Type converter for the CustomStateButtonTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomStateButtonToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomStateButtonTool).GetConstructor(New Type() {GetType(String)})
                    Dim StateButtonTool As CustomStateButtonTool = CType(value, CustomStateButtonTool)

                    Return New InstanceDescriptor(ci, New Object() {StateButtonTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomStateButtonTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomStateButtonToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomStateButtonToolExtraSharedProps = New CustomStateButtonToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomStateButtonToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomStateButtonToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomStateButtonToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomStateButtonTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomStateButtonTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomStateButtonTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomStateButtonTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomStateButtonTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomStateButtonTool Class

#Region " CustomTextboxTool Class "
    'CustomTextboxTool Class
    <TypeConverter(GetType(CustomTextboxTool.CustomTextboxToolConverter)), _
    Serializable()> _
    Public Class CustomTextboxTool
        Inherits TextBoxTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomTextboxTool.CustomTextboxToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomTextboxTool = New CustomTextboxTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "
        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        <Browsable(False)>
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomTextboxTool.CustomTextboxToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomTextboxTool.CustomTextboxToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomTextboxTool Type Converter "
        'Type converter for the CustomTextboxTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomTextboxToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomTextboxTool).GetConstructor(New Type() {GetType(String)})
                    Dim TextboxTool As CustomTextboxTool = CType(value, CustomTextboxTool)

                    Return New InstanceDescriptor(ci, New Object() {TextboxTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomTextboxTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomTextboxToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomTextboxToolExtraSharedProps = New CustomTextboxToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomTextboxToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomTextboxToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomTextboxToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomTextboxTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomTextboxTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomTextboxTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomTextboxTool.DEFAULTVALUE_NABITABINDEXPROP

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomTextboxTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties
    End Class

#End Region  'CustomTextboxTool Class

End Namespace
